@@ -2,7 +2,7 @@
  *
  * $Id: groupdef.cpp,v 1.29 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2008 by Dimitri van Heesch.
+ * Copyright (C) 1997-2010 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -35,6 +35,9 @@
 #include "searchindex.h"
 #include "dot.h"
 #include "vhdldocgen.h"
+#include "layout.h"
+
+//---------------------------------------------------------------------------
 
 GroupDef::GroupDef(const char *df,int dl,const char *na,const char *t,
                    const char *refFileName) : Definition(df,dl,na)
@@ -60,24 +63,6 @@ GroupDef::GroupDef(const char *df,int dl,const char *na,const char *t,
   memberGroupSDict->setAutoDelete(TRUE);
 
   allMemberList = new MemberList(MemberList::allMembersList);
-#if 0
-
-  decDefineMembers.setInGroup(TRUE);
-  decProtoMembers.setInGroup(TRUE);
-  decTypedefMembers.setInGroup(TRUE);
-  decEnumMembers.setInGroup(TRUE);
-  decEnumValMembers.setInGroup(TRUE);
-  decFuncMembers.setInGroup(TRUE);
-  decVarMembers.setInGroup(TRUE);
-
-  docDefineMembers.setInGroup(TRUE);
-  docProtoMembers.setInGroup(TRUE);
-  docTypedefMembers.setInGroup(TRUE);
-  docEnumMembers.setInGroup(TRUE);
-  docEnumValMembers.setInGroup(TRUE);
-  docFuncMembers.setInGroup(TRUE);
-  docVarMembers.setInGroup(TRUE);
-#endif
 
   visited = 0;
   groupScope = 0;
@@ -142,14 +127,6 @@ void GroupDef::findSectionsInDocumentation()
       ml->findSectionsInDocumentation();
     }
   }
-#if 0
-  decDefineMembers.findSectionsInDocumentation();
-  decProtoMembers.findSectionsInDocumentation();
-  decTypedefMembers.findSectionsInDocumentation();
-  decEnumMembers.findSectionsInDocumentation();
-  decFuncMembers.findSectionsInDocumentation();
-  decVarMembers.findSectionsInDocumentation();
-#endif
 }
 
 void GroupDef::addFile(const FileDef *def)
@@ -224,15 +201,6 @@ void GroupDef::addMembersToMemberGroup()
       ::addMembersToMemberGroup(ml,&memberGroupSDict,this);
     }
   }
-#if 0
-  ::addMembersToMemberGroup(&decDefineMembers,&memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decProtoMembers,&memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decTypedefMembers,&memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decEnumMembers,&memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decEnumValMembers,&memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decFuncMembers,&memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decVarMembers,&memberGroupSDict,this);
-#endif
 
   //printf("GroupDef::addMembersToMemberGroup() memberGroupList=%d\n",memberGroupList->count());
   MemberGroupSDict::Iterator mgli(*memberGroupSDict);
@@ -332,13 +300,6 @@ bool GroupDef::insertMember(MemberDef *md,bool docOnly)
       }
       addMemberToList(MemberList::docEnumValMembers,md);
       break;
-    case MemberDef::Prototype:    
-      if (!docOnly)
-      {
-        addMemberToList(MemberList::decProtoMembers,md);
-      }
-      addMemberToList(MemberList::docProtoMembers,md);
-      break;
     case MemberDef::Define:       
       if (!docOnly)
       {
@@ -455,10 +416,6 @@ void GroupDef::removeMember(MemberDef *md)
         removeMemberFromList(MemberList::decEnumValMembers,md);
         removeMemberFromList(MemberList::docEnumValMembers,md);
         break;
-      case MemberDef::Prototype:    
-        removeMemberFromList(MemberList::decProtoMembers,md);
-        removeMemberFromList(MemberList::docProtoMembers,md);
-        break;
       case MemberDef::Define:       
         removeMemberFromList(MemberList::decDefineMembers,md);
         removeMemberFromList(MemberList::docDefineMembers,md);
@@ -540,64 +497,91 @@ void GroupDef::computeAnchors()
   setAnchors(0,'a',allMemberList);
 }
 
-void GroupDef::writeDetailedDocumentation(OutputList &ol)
+void GroupDef::writeDetailedDescription(OutputList &ol,const QCString &title)
 {
-  if (!briefDescription().isEmpty() || !documentation().isEmpty())
+  if ((!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF")) 
+      || !documentation().isEmpty() || !inbodyDocumentation().isEmpty()
+     )
   {
-    
-    if (pageDict->count()!=countMembers()) // classical layout
+    if (pageDict->count()!=countMembers()) // not only pages -> classical layout
     {
       ol.writeRuler();
       ol.pushGeneratorState();
-      ol.disable(OutputGenerator::Latex);
-      ol.disable(OutputGenerator::RTF);
-      ol.writeAnchor(0,"_details");
+      ol.disableAllBut(OutputGenerator::Html);
+        ol.writeAnchor(0,"_details");
       ol.popGeneratorState();
       ol.startGroupHeader();
-      ol.parseText(theTranslator->trDetailedDescription());
+      ol.parseText(title);
       ol.endGroupHeader();
-
-      // repeat brief description
-      if (!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF"))
-      {
-        ol.parseDoc(briefFile(),briefLine(),this,0,briefDescription(),FALSE,FALSE);
-        ol.newParagraph();
-      }
     }
 
-    // write documentation
+    // repeat brief description
+    if (!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF"))
+    {
+      ol.parseDoc(briefFile(),briefLine(),this,0,briefDescription(),FALSE,FALSE);
+    }
+    // write separator between brief and details
+    if (!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF") &&
+        !documentation().isEmpty())
+    {
+      ol.pushGeneratorState();
+      ol.disable(OutputGenerator::Man);
+      ol.disable(OutputGenerator::RTF);
+      // ol.newParagraph(); // FIXME:PARA
+      ol.enableAll();
+      ol.disableAllBut(OutputGenerator::Man);
+      ol.writeString("\n\n");
+      ol.popGeneratorState();
+    }
+
+    // write detailed documentation
     if (!documentation().isEmpty())
     {
       ol.parseDoc(docFile(),docLine(),this,0,documentation()+"\n",TRUE,FALSE);
     }
+
+    // write inbody documentation
+    if (!inbodyDocumentation().isEmpty())
+    {
+      ol.parseDoc(inbodyFile(),inbodyLine(),this,0,inbodyDocumentation()+"\n",TRUE,FALSE);
+    }
   }
 }
 
-
-void GroupDef::writeDocumentation(OutputList &ol)
+void GroupDef::writeBriefDescription(OutputList &ol)
 {
-  bool fortranOpt = Config_getBool("OPTIMIZE_FOR_FORTRAN");  
-  bool vhdlOpt    = Config_getBool("OPTIMIZE_OUTPUT_VHDL");  
-  ol.pushGeneratorState();
-  startFile(ol,getOutputFileBase(),name(),title);
-  startTitle(ol,getOutputFileBase());
-  ol.parseText(title);
-  addGroupListToTitle(ol,this);
-  endTitle(ol,getOutputFileBase(),title);
-
-  if (Config_getBool("SEARCHENGINE"))
+  if (!briefDescription().isEmpty())
   {
-    Doxygen::searchIndex->setCurrentDoc(title,getOutputFileBase());
-    static QRegExp we("[a-zA-Z_][a-zA-Z_0-9]*");
-    int i=0,p=0,l=0;
-    while ((i=we.match(title,p,&l))!=-1) // foreach word in the title
-    {
-      Doxygen::searchIndex->addWord(title.mid(i,l),TRUE);
-      p=i+l;
-    }
-  }
+    ol.startParagraph();
+    ol.parseDoc(briefFile(),briefLine(),this,0,
+                briefDescription(),TRUE,FALSE,0,TRUE,FALSE);
+    ol.pushGeneratorState();
+    ol.disable(OutputGenerator::RTF);
+    ol.writeString(" \n");
+    ol.enable(OutputGenerator::RTF);
 
-  if (Config_getBool("HAVE_DOT") && Config_getBool("GROUP_GRAPHS") )
+    if (Config_getBool("REPEAT_BRIEF") ||
+        !documentation().isEmpty()
+       )
+    {
+      ol.disableAllBut(OutputGenerator::Html);
+      ol.startTextLink(0,"_details");
+      ol.parseText(theTranslator->trMore());
+      ol.endTextLink();
+    }
+    ol.popGeneratorState();
+
+    //ol.pushGeneratorState();
+    //ol.disable(OutputGenerator::RTF);
+    //ol.newParagraph();
+    //ol.popGeneratorState();
+    ol.endParagraph();
+  }
+}
+
+void GroupDef::writeGroupGraph(OutputList &ol)
+{
+  if (Config_getBool("HAVE_DOT") /*&& Config_getBool("GROUP_GRAPHS")*/ )
   {
     DotGroupCollaboration graph(this);
     if (!graph.isTrivial())
@@ -605,52 +589,23 @@ void GroupDef::writeDocumentation(OutputList &ol)
       msg("Generating dependency graph for group %s\n",qualifiedName().data());
       ol.pushGeneratorState();
       ol.disable(OutputGenerator::Man);
-      ol.newParagraph();
+      ol.startParagraph();
       ol.startGroupCollaboration();
       ol.parseText(theTranslator->trCollaborationDiagram(title));
       ol.endGroupCollaboration(graph);
+      ol.endParagraph();
       ol.popGeneratorState();
     }
   }
+}
 
-
-  if (Config_getBool("DETAILS_AT_TOP"))
-  {
-    writeDetailedDocumentation(ol);
-    ol.newParagraph();
-  }
-  else if (!briefDescription().isEmpty())
-  {
-    ol.parseDoc(briefFile(),briefLine(),this,0,briefDescription(),TRUE,FALSE);
-    ol.writeString(" \n");
-    ol.pushGeneratorState();
-    ol.disable(OutputGenerator::Latex);
-    ol.disable(OutputGenerator::RTF);
-    ol.disable(OutputGenerator::Man);
-    ol.startTextLink(0,"_details");
-    ol.parseText(theTranslator->trMore());
-    ol.endTextLink();
-    ol.enableAll();
-    ol.disableAllBut(OutputGenerator::Man);
-    ol.newParagraph();
-    ol.popGeneratorState();
-  }
-
-  if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
-  {
-    Doxygen::tagFile << "  <compound kind=\"group\">" << endl;
-    Doxygen::tagFile << "    <name>" << convertToXML(name()) << "</name>" << endl;
-    Doxygen::tagFile << "    <title>" << convertToXML(title) << "</title>" << endl;
-    Doxygen::tagFile << "    <filename>" << convertToXML(getOutputFileBase()) << Doxygen::htmlFileExtension << "</filename>" << endl;
-  }
-  
-  ol.startMemberSections();
-
+void GroupDef::writeFiles(OutputList &ol,const QCString &title)
+{
   // write list of files
   if (fileList->count()>0)
   {
-    ol.startMemberHeader();
-    ol.parseText(theTranslator->trFile(TRUE,FALSE));
+    ol.startMemberHeader("files");
+    ol.parseText(title);
     ol.endMemberHeader();
     ol.startMemberList();
     FileDef *fd=fileList->first();
@@ -667,24 +622,31 @@ void GroupDef::writeDocumentation(OutputList &ol)
       ol.endMemberItem();
       if (!fd->briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
       {
+        ol.startParagraph();
         ol.startMemberDescription();
         ol.parseDoc(briefFile(),briefLine(),fd,0,fd->briefDescription(),FALSE,FALSE);
         ol.endMemberDescription();
-        ol.newParagraph();
+        ol.endParagraph();
       }
       fd=fileList->next();
     }
     ol.endMemberList();
   }
+}
 
+void GroupDef::writeNamespaces(OutputList &ol,const QCString &title)
+{
   // write list of namespaces
-  namespaceSDict->writeDeclaration(ol);
+  namespaceSDict->writeDeclaration(ol,title);
+}
 
+void GroupDef::writeNestedGroups(OutputList &ol,const QCString &title)
+{
   // write list of groups
   if (groupList->count()>0)
   {
-    ol.startMemberHeader();
-    ol.parseText(theTranslator->trModules());
+    ol.startMemberHeader("groups");
+    ol.parseText(title);
     ol.endMemberHeader();
     ol.startMemberList();
     GroupDef *gd=groupList->first();
@@ -702,21 +664,25 @@ void GroupDef::writeDocumentation(OutputList &ol)
       ol.endMemberItem();
       if (!gd->briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
       {
+        ol.startParagraph();
         ol.startMemberDescription();
         ol.parseDoc(briefFile(),briefLine(),gd,0,gd->briefDescription(),FALSE,FALSE);
         ol.endMemberDescription();
-        ol.newParagraph();
+        ol.endParagraph();
       }
       gd=groupList->next();
     }
     ol.endMemberList();
   }
+}
 
+void GroupDef::writeDirs(OutputList &ol,const QCString &title)
+{
   // write list of directories
   if (dirList->count()>0)
   {
-    ol.startMemberHeader();
-    ol.parseText(theTranslator->trDirectories());
+    ol.startMemberHeader("dirs");
+    ol.parseText(title);
     ol.endMemberHeader();
     ol.startMemberList();
     DirDef *dd=dirList->first();
@@ -733,56 +699,27 @@ void GroupDef::writeDocumentation(OutputList &ol)
       }
       if (!dd->briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
       {
+        ol.startParagraph();
         ol.startMemberDescription();
         ol.parseDoc(briefFile(),briefLine(),dd,0,dd->briefDescription(),FALSE,FALSE);
         ol.endMemberDescription();
-        ol.newParagraph();
+        ol.endParagraph();
       }
       dd=dirList->next();
     }
 
     ol.endMemberList();
   }
-  
+}
+
+void GroupDef::writeClasses(OutputList &ol,const QCString &title)
+{
   // write list of classes
-  classSDict->writeDeclaration(ol);
-
-  // write list of members
-  if (allMemberList->count()>0)
-  {
-    /* write user defined member groups */
-    MemberGroupSDict::Iterator mgli(*memberGroupSDict);
-    MemberGroup *mg;
-    for (;(mg=mgli.current());++mgli)
-    {
-      mg->writeDeclarations(ol,0,0,0,this);
-    }
-
-    writeMemberDeclarations(ol,MemberList::decDefineMembers,theTranslator->trDefines());
-    writeMemberDeclarations(ol,MemberList::decProtoMembers,theTranslator->trFuncProtos());
-    writeMemberDeclarations(ol,MemberList::decTypedefMembers,theTranslator->trTypedefs());
-    writeMemberDeclarations(ol,MemberList::decEnumMembers,theTranslator->trEnumerations());
-    writeMemberDeclarations(ol,MemberList::decEnumValMembers,theTranslator->trEnumerationValues());
-    writeMemberDeclarations(ol,MemberList::decFuncMembers,
-                 fortranOpt ? theTranslator->trSubprograms()  : 
-                 vhdlOpt    ? VhdlDocGen::trFunctionAndProc() :
-                              theTranslator->trFunctions());
-    writeMemberDeclarations(ol,MemberList::decVarMembers,theTranslator->trVariables());
-    writeMemberDeclarations(ol,MemberList::decSignalMembers,theTranslator->trSignals());
-    writeMemberDeclarations(ol,MemberList::decPubSlotMembers,theTranslator->trPublicSlots());
-    writeMemberDeclarations(ol,MemberList::decProSlotMembers,theTranslator->trProtectedSlots());
-    writeMemberDeclarations(ol,MemberList::decPriSlotMembers,theTranslator->trPrivateSlots());
-    writeMemberDeclarations(ol,MemberList::decEventMembers,theTranslator->trEvents());
-    writeMemberDeclarations(ol,MemberList::decPropMembers,theTranslator->trProperties());
-    writeMemberDeclarations(ol,MemberList::decFriendMembers,theTranslator->trFriends());
-  }
-  ol.endMemberSections();
-
-  if (!Config_getBool("DETAILS_AT_TOP"))
-  {
-    writeDetailedDocumentation(ol);
-  }
+  classSDict->writeDeclaration(ol,0,title,FALSE);
+}
 
+void GroupDef::writePageDocumentation(OutputList &ol)
+{
   PageDef *pd=0;
   PageSDict::Iterator pdi(*pageDict);
   for (pdi.toFirst();(pd=pdi.current());++pdi)
@@ -805,12 +742,253 @@ void GroupDef::writeDocumentation(OutputList &ol)
         ol.endSection(si->label,SectionInfo::Subsection);
       }
       ol.startTextBlock();
-      ol.parseDoc(pd->docFile(),pd->docLine(),pd,0,pd->documentation(),TRUE,FALSE);
+      ol.parseDoc(pd->docFile(),pd->docLine(),pd,0,pd->documentation()+pd->inbodyDocumentation(),TRUE,FALSE);
       ol.endTextBlock();
     }
   }
+}
 
-  writeMemberDocumentation(ol);
+void GroupDef::writeMemberGroups(OutputList &ol)
+{
+  /* write user defined member groups */
+  if (memberGroupSDict)
+  {
+    /* write user defined member groups */
+    MemberGroupSDict::Iterator mgli(*memberGroupSDict);
+    MemberGroup *mg;
+    for (;(mg=mgli.current());++mgli)
+    {
+      mg->writeDeclarations(ol,0,0,0,this);
+    }
+  }
+}
+
+void GroupDef::startMemberDeclarations(OutputList &ol)
+{
+  ol.startMemberSections();
+}
+
+void GroupDef::endMemberDeclarations(OutputList &ol)
+{
+  ol.endMemberSections();
+}
+
+void GroupDef::startMemberDocumentation(OutputList &ol)
+{
+  if (Config_getBool("SEPARATE_MEMBER_PAGES"))
+  {
+    ol.disable(OutputGenerator::Html);
+    Doxygen::suppressDocWarnings = TRUE;
+  }
+}
+
+void GroupDef::endMemberDocumentation(OutputList &ol)
+{
+  if (Config_getBool("SEPARATE_MEMBER_PAGES"))
+  {
+    ol.enable(OutputGenerator::Html);
+    Doxygen::suppressDocWarnings = FALSE;
+  }
+}
+
+void GroupDef::writeAuthorSection(OutputList &ol)
+{
+  // write Author section (Man only)
+  ol.pushGeneratorState();
+  ol.disableAllBut(OutputGenerator::Man);
+  ol.startGroupHeader();
+  ol.parseText(theTranslator->trAuthor(TRUE,TRUE));
+  ol.endGroupHeader();
+  ol.parseText(theTranslator->trGeneratedAutomatically(Config_getString("PROJECT_NAME")));
+  ol.popGeneratorState();
+}
+
+void GroupDef::writeSummaryLinks(OutputList &ol)
+{
+  ol.pushGeneratorState();
+  ol.disableAllBut(OutputGenerator::Html);
+  QListIterator<LayoutDocEntry> eli(
+      LayoutDocManager::instance().docEntries(LayoutDocManager::Group));
+  LayoutDocEntry *lde;
+  bool first=TRUE;
+  for (eli.toFirst();(lde=eli.current());++eli)
+  {
+    if ((lde->kind()==LayoutDocEntry::GroupClasses && classSDict->declVisible()) || 
+        (lde->kind()==LayoutDocEntry::GroupNamespaces && namespaceSDict->declVisible()) ||
+        (lde->kind()==LayoutDocEntry::GroupFiles && fileList->count()>0) ||
+        (lde->kind()==LayoutDocEntry::GroupNestedGroups && groupList->count()>0) ||
+        (lde->kind()==LayoutDocEntry::GroupDirs && dirList->count()>0)
+       )
+    {
+      LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
+      QCString label = lde->kind()==LayoutDocEntry::GroupClasses      ? "nested-classes" : 
+                       lde->kind()==LayoutDocEntry::GroupNamespaces   ? "namespaces"     :
+                       lde->kind()==LayoutDocEntry::GroupFiles        ? "files"          :
+                       lde->kind()==LayoutDocEntry::GroupNestedGroups ? "groups"         :
+                       "dirs";
+      writeSummaryLink(ol,label,ls->title,first);
+    }
+    else if (lde->kind()==LayoutDocEntry::MemberDecl)
+    {
+      LayoutDocEntryMemberDecl *lmd = (LayoutDocEntryMemberDecl*)lde;
+      MemberList * ml = getMemberList(lmd->type);
+      if (ml && ml->declVisible())
+      {
+        writeSummaryLink(ol,ml->listTypeAsString(),lmd->title,first);
+      }
+    }
+  }
+  if (!first)
+  {
+    ol.writeString("  </div>\n");
+  }
+  ol.popGeneratorState();
+}
+
+void GroupDef::writeDocumentation(OutputList &ol)
+{
+  ol.pushGeneratorState();
+  startFile(ol,getOutputFileBase(),name(),title);
+  startTitle(ol,getOutputFileBase(),this);
+  ol.parseText(title);
+  addGroupListToTitle(ol,this);
+  endTitle(ol,getOutputFileBase(),title);
+  ol.startContents();
+
+  if (Doxygen::searchIndex)
+  {
+    Doxygen::searchIndex->setCurrentDoc(title,getOutputFileBase());
+    static QRegExp we("[a-zA-Z_][-a-zA-Z_0-9]*");
+    int i=0,p=0,l=0;
+    while ((i=we.match(title,p,&l))!=-1) // foreach word in the title
+    {
+      Doxygen::searchIndex->addWord(title.mid(i,l),TRUE);
+      p=i+l;
+    }
+  }
+
+  Doxygen::indexList.addIndexItem(this,0,0,title);
+
+  if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
+  {
+    Doxygen::tagFile << "  <compound kind=\"group\">" << endl;
+    Doxygen::tagFile << "    <name>" << convertToXML(name()) << "</name>" << endl;
+    Doxygen::tagFile << "    <title>" << convertToXML(title) << "</title>" << endl;
+    Doxygen::tagFile << "    <filename>" << convertToXML(getOutputFileBase()) << Doxygen::htmlFileExtension << "</filename>" << endl;
+  }
+  
+
+  //---------------------------------------- start flexible part -------------------------------
+
+  QListIterator<LayoutDocEntry> eli(
+      LayoutDocManager::instance().docEntries(LayoutDocManager::Group));
+  LayoutDocEntry *lde;
+  for (eli.toFirst();(lde=eli.current());++eli)
+  {
+    switch (lde->kind())
+    {
+      case LayoutDocEntry::BriefDesc: 
+        writeBriefDescription(ol);
+        break; 
+      case LayoutDocEntry::MemberDeclStart: 
+        startMemberDeclarations(ol);
+        break; 
+      case LayoutDocEntry::GroupClasses: 
+        {
+          LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
+          writeClasses(ol,ls->title);
+        }
+        break; 
+      case LayoutDocEntry::GroupNamespaces: 
+        {
+          LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
+          writeNamespaces(ol,ls->title);
+        }
+        break; 
+      case LayoutDocEntry::MemberGroups: 
+        writeMemberGroups(ol);
+        break; 
+      case LayoutDocEntry::MemberDecl: 
+        {
+          LayoutDocEntryMemberDecl *lmd = (LayoutDocEntryMemberDecl*)lde;
+          writeMemberDeclarations(ol,lmd->type,lmd->title);
+        }
+        break; 
+      case LayoutDocEntry::MemberDeclEnd: 
+        endMemberDeclarations(ol);
+        break;
+      case LayoutDocEntry::DetailedDesc: 
+        {
+          LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
+          writeDetailedDescription(ol,ls->title);
+        }
+        break;
+      case LayoutDocEntry::MemberDefStart: 
+        startMemberDocumentation(ol);
+        break; 
+      case LayoutDocEntry::MemberDef: 
+        {
+          LayoutDocEntryMemberDef *lmd = (LayoutDocEntryMemberDef*)lde;
+          writeMemberDocumentation(ol,lmd->type,lmd->title);
+        }
+        break;
+      case LayoutDocEntry::MemberDefEnd: 
+        endMemberDocumentation(ol);
+        break;
+      case LayoutDocEntry::GroupNestedGroups: 
+        {
+          LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
+          writeNestedGroups(ol,ls->title);
+        }
+        break;
+      case LayoutDocEntry::GroupPageDocs: 
+        writePageDocumentation(ol);
+        break;
+      case LayoutDocEntry::GroupDirs: 
+        {
+          LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
+          writeDirs(ol,ls->title);
+        }
+        break;
+      case LayoutDocEntry::GroupFiles: 
+        {
+          LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
+          writeFiles(ol,ls->title);
+        }
+        break;
+      case LayoutDocEntry::GroupGraph: 
+        writeGroupGraph(ol);
+        break;
+      case LayoutDocEntry::AuthorSection: 
+        writeAuthorSection(ol);
+        break;
+      case LayoutDocEntry::ClassIncludes:
+      case LayoutDocEntry::ClassInheritanceGraph:
+      case LayoutDocEntry::ClassNestedClasses:
+      case LayoutDocEntry::ClassCollaborationGraph:
+      case LayoutDocEntry::ClassAllMembersLink:
+      case LayoutDocEntry::ClassUsedFiles:
+      case LayoutDocEntry::NamespaceNestedNamespaces:
+      case LayoutDocEntry::NamespaceClasses:
+      case LayoutDocEntry::FileClasses:
+      case LayoutDocEntry::FileNamespaces:
+      case LayoutDocEntry::FileIncludes:
+      case LayoutDocEntry::FileIncludeGraph:
+      case LayoutDocEntry::FileIncludedByGraph: 
+      case LayoutDocEntry::FileSourceLink:
+      case LayoutDocEntry::DirSubDirs:
+      case LayoutDocEntry::DirFiles:
+      case LayoutDocEntry::DirGraph:
+        err("Internal inconsistency: member %d should not be part of "
+            "LayoutDocManager::Group entry list\n",lde->kind());
+        break;
+    }
+  }
+
+  //---------------------------------------- end flexible part -------------------------------
+
+  endFile(ol); 
+  ol.popGeneratorState();
 
   if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
   {
@@ -818,9 +996,6 @@ void GroupDef::writeDocumentation(OutputList &ol)
     Doxygen::tagFile << "  </compound>" << endl;
   }
 
-  endFile(ol); 
-  ol.popGeneratorState();
-
   if (Config_getBool("SEPARATE_MEMBER_PAGES"))
   {
     allMemberList->sort();
@@ -829,35 +1004,6 @@ void GroupDef::writeDocumentation(OutputList &ol)
 
 }
 
-void GroupDef::writeMemberDocumentation(OutputList &ol)
-{
-  bool  fortranOpt = Config_getBool("OPTIMIZE_FOR_FORTRAN");
-  if (Config_getBool("SEPARATE_MEMBER_PAGES"))
-  {
-    ol.disable(OutputGenerator::Html);
-  }
-  
-  writeMemberDocumentation(ol,MemberList::docDefineMembers,theTranslator->trDefineDocumentation());
-  writeMemberDocumentation(ol,MemberList::docProtoMembers,theTranslator->trFunctionPrototypeDocumentation());
-  writeMemberDocumentation(ol,MemberList::docTypedefMembers,theTranslator->trTypedefDocumentation());
-  writeMemberDocumentation(ol,MemberList::docEnumMembers,theTranslator->trEnumerationTypeDocumentation());
-  writeMemberDocumentation(ol,MemberList::docEnumValMembers,theTranslator->trEnumerationValueDocumentation());
-  writeMemberDocumentation(ol,MemberList::docFuncMembers,fortranOpt?theTranslator->trSubprogramDocumentation():theTranslator->trFunctionDocumentation());
-  writeMemberDocumentation(ol,MemberList::docVarMembers,theTranslator->trVariableDocumentation());
-  writeMemberDocumentation(ol,MemberList::docSignalMembers,theTranslator->trSignals());   // todo: add trSignalDocumentation()
-  writeMemberDocumentation(ol,MemberList::docPubSlotMembers,theTranslator->trPublicSlots()); // todo: add trSlotDocumentation()
-  writeMemberDocumentation(ol,MemberList::docProSlotMembers,theTranslator->trProtectedSlots()); // todo: add trSlotDocumentation()
-  writeMemberDocumentation(ol,MemberList::docPriSlotMembers,theTranslator->trPrivateSlots()); // todo: add trSlotDocumentation()
-  writeMemberDocumentation(ol,MemberList::docEventMembers,theTranslator->trEvents());     // todo: add trEventDocumentation()
-  writeMemberDocumentation(ol,MemberList::docPropMembers,theTranslator->trProperties());  // todo: add trPropertyDocumentation()
-  writeMemberDocumentation(ol,MemberList::docFriendMembers,theTranslator->trFriends());   // todo: add trFriendDocumentation()
-
-  if (Config_getBool("SEPARATE_MEMBER_PAGES"))
-  {
-    ol.enable(OutputGenerator::Html);
-  }
-}
-
 void GroupDef::writeMemberPages(OutputList &ol)
 {
   ol.pushGeneratorState();
@@ -872,14 +1018,6 @@ void GroupDef::writeMemberPages(OutputList &ol)
        ml->writeDocumentationPage(ol,name(),this);
     }
   }
-#if 0
-  docDefineMembers.writeDocumentationPage(ol,name(),this);
-  docProtoMembers.writeDocumentationPage(ol,name(),this);
-  docTypedefMembers.writeDocumentationPage(ol,name(),this);
-  docEnumMembers.writeDocumentationPage(ol,name(),this);
-  docFuncMembers.writeDocumentationPage(ol,name(),this);
-  docVarMembers.writeDocumentationPage(ol,name(),this);
-#endif
 
   ol.popGeneratorState();
 }
@@ -1021,7 +1159,7 @@ void addMemberToGroups(Entry *root,MemberDef *md)
       if (fgd && gd!=fgd && g->pri==pri) 
       {
          warn(root->fileName.data(), root->startLine,
-           "Warning: Member %s found in multiple %s groups! "
+           "warning: Member %s found in multiple %s groups! "
            "The member will be put in group %s, and not in group %s",
 	   md->name().data(), Grouping::getGroupPriName( pri ),
 	   gd->name().data(), fgd->name().data()
@@ -1066,7 +1204,7 @@ void addMemberToGroups(Entry *root,MemberDef *md)
           else if (!root->doc.isEmpty() && md->getGroupHasDocs())
           {
             warn(md->getGroupFileName(),md->getGroupStartLine(),
-                "Warning: Member documentation for %s found several times in %s groups!\n"
+                "warning: Member documentation for %s found several times in %s groups!\n"
                 "%s:%d: The member will remain in group %s, and won't be put into group %s",
                 md->name().data(), Grouping::getGroupPriName( pri ),
                 root->fileName.data(), root->startLine,
@@ -1135,8 +1273,10 @@ void GroupDef::addListReferences()
   {
     LockingPtr< QList<ListItemInfo> > xrefItems = xrefListItems();
     addRefItem(xrefItems.pointer(),
+             getOutputFileBase(),
              theTranslator->trGroup(TRUE,TRUE),
-             getOutputFileBase(),name()
+             getOutputFileBase(),name(),
+             0
             );
   }
   MemberGroupSDict::Iterator mgli(*memberGroupSDict);
@@ -1154,14 +1294,6 @@ void GroupDef::addListReferences()
       ml->addListReferences(this);
     }
   }
-#if 0
-  docDefineMembers.addListReferences(this);
-  docProtoMembers.addListReferences(this);
-  docTypedefMembers.addListReferences(this);
-  docEnumMembers.addListReferences(this);
-  docFuncMembers.addListReferences(this);
-  docVarMembers.addListReferences(this);
-#endif
 }
 
 MemberList *GroupDef::createMemberList(MemberList::ListType lt)
@@ -1188,14 +1320,30 @@ void GroupDef::addMemberToList(MemberList::ListType lt,MemberDef *md)
   static bool sortBriefDocs = Config_getBool("SORT_BRIEF_DOCS");
   static bool sortMemberDocs = Config_getBool("SORT_MEMBER_DOCS");
   MemberList *ml = createMemberList(lt);
-  if (((ml->listType()&MemberList::declarationLists) && sortBriefDocs) ||
-      ((ml->listType()&MemberList::documentationLists) && sortMemberDocs)
-     )
+  ml->setNeedsSorting(
+      ((ml->listType()&MemberList::declarationLists) && sortBriefDocs) ||
+      ((ml->listType()&MemberList::documentationLists) && sortMemberDocs));
+  ml->append(md);
+
+#if 0  
+  if (ml->needsSorting())
     ml->inSort(md);
   else
     ml->append(md);
+#endif
 }
 
+void GroupDef::sortMemberLists()
+{
+  MemberList *ml = m_memberLists.first();
+  while (ml)
+  {
+    if (ml->needsSorting()) { ml->sort(); ml->setNeedsSorting(FALSE); }
+    ml = m_memberLists.next();
+  }
+}
+
+
 MemberList *GroupDef::getMemberList(MemberList::ListType lt) const
 {
   GroupDef *that = (GroupDef*)this;
@@ -1213,8 +1361,18 @@ MemberList *GroupDef::getMemberList(MemberList::ListType lt) const
 
 void GroupDef::writeMemberDeclarations(OutputList &ol,MemberList::ListType lt,const QCString &title)
 {
+  static bool optimizeVhdl = Config_getBool("OPTIMIZE_OUTPUT_VHDL");
+
   MemberList * ml = getMemberList(lt);
-  if (ml) ml->writeDeclarations(ol,0,0,0,this,title,0);
+  if (optimizeVhdl && ml) 
+  {
+    VhdlDocGen::writeVhdlDeclarations(ml,ol,this,0,0);
+    return;
+  }
+  if (ml) 
+  {
+    ml->writeDeclarations(ol,0,0,0,this,title,0);
+  }
 }
 
 void GroupDef::writeMemberDocumentation(OutputList &ol,MemberList::ListType lt,const QCString &title)
