@@ -2,7 +2,7 @@
  *
  * $Id: index.h,v 1.28 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2007 by Dimitri van Heesch.
+ * Copyright (C) 1997-2010 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -22,10 +22,12 @@
 #include <qfile.h>
 #include <qlist.h>
 
+class Definition;
 class MemberDef;
 class OutputList;
+class FTextStream;
 
-
+/** \brief Abstract interface for index generators. */
 class IndexIntf
 {
   public:
@@ -36,35 +38,60 @@ class IndexIntf
     virtual void decContentsDepth() = 0;
     virtual void addContentsItem(bool isDir, const char *name, const char *ref = 0, 
                                  const char *file = 0, const char *anchor = 0) = 0;
-    virtual void addIndexItem(const char *level1, const char *level2, const char *contRef, 
-                              const char *memRef, const char *anchor) = 0;
+    virtual void addIndexItem(Definition *context,MemberDef *md,
+                              const char *anchor,const char *word) = 0;
     virtual void addIndexFile(const char *name) = 0;
+    virtual void addImageFile(const char *name) = 0;
+    virtual void addStyleSheetFile(const char *name) = 0;
 };
 
+/** \brief A list of index interfaces.
+ *
+ *  This class itself implements all methods of IndexIntf and
+ *  just forwards the calls to all items in the list.
+ */
 class IndexList : public IndexIntf
 {
+  private:
+    QList<IndexIntf> m_intfs;
+
+    // --- foreach implementations for various number of arguments
+
     void foreach(void (IndexIntf::*methodPtr)())
     {
       QListIterator<IndexIntf> li(m_intfs);
       for (li.toFirst();li.current();++li) (li.current()->*methodPtr)();
-    };
+    }
+
     template<typename A1>
     void foreach(void (IndexIntf::*methodPtr)(A1),A1 a1)
     {
       QListIterator<IndexIntf> li(m_intfs);
       for (li.toFirst();li.current();++li) (li.current()->*methodPtr)(a1);
-    };
+    }
+
+    template<typename A1,typename A2,typename A3,typename A4>
+    void foreach(void (IndexIntf::*methodPtr)(A1,A2,A3,A4),A1 a1,A2 a2,A3 a3,A4 a4)
+    {
+      QListIterator<IndexIntf> li(m_intfs);
+      for (li.toFirst();li.current();++li) (li.current()->*methodPtr)(a1,a2,a3,a4);
+    }
+
     template<typename A1,typename A2,typename A3,typename A4,typename A5>
     void foreach(void (IndexIntf::*methodPtr)(A1,A2,A3,A4,A5),A1 a1,A2 a2,A3 a3,A4 a4,A5 a5)
     {
       QListIterator<IndexIntf> li(m_intfs);
       for (li.toFirst();li.current();++li) (li.current()->*methodPtr)(a1,a2,a3,a4,a5);
-    };
+    }
 
   public:
+    /** Creates a list of indexes */
     IndexList() { m_intfs.setAutoDelete(TRUE); }
+    /** Add an index generator to the list */
     void addIndex(IndexIntf *intf) 
     { m_intfs.append(intf); }
+
+    // IndexIntf implementation
     void initialize() 
     { foreach(&IndexIntf::initialize); }
     void finalize() 
@@ -77,15 +104,17 @@ class IndexList : public IndexIntf
                          const char *file = 0, const char *anchor = 0)
     { foreach<bool,const char *,const char *,const char *,const char*>
              (&IndexIntf::addContentsItem,isDir,name,ref,file,anchor); }
-    void addIndexItem(const char *level1, const char *level2, const char *contRef, 
-                      const char *memRef, const char *anchor)
-    { foreach<const char *,const char *,const char *,const char *,const char *>
-             (&IndexIntf::addIndexItem,level1,level2,contRef,memRef,anchor); }
+    void addIndexItem(Definition *context,MemberDef *md,
+                      const char *anchor=0,const char *word=0)
+    { foreach<Definition *,MemberDef *>
+             (&IndexIntf::addIndexItem,context,md,anchor,word); }
     void addIndexFile(const char *name) 
     { foreach<const char *>(&IndexIntf::addIndexFile,name); }
+    void addImageFile(const char *name) 
+    { foreach<const char *>(&IndexIntf::addImageFile,name); }
+    void addStyleSheetFile(const char *name) 
+    { foreach<const char *>(&IndexIntf::addStyleSheetFile,name); }
 
-  private:
-    QList<IndexIntf> m_intfs;
 };
 
 
@@ -108,6 +137,7 @@ enum IndexSections
   isFileDocumentation,
   isExampleDocumentation,
   isPageDocumentation,
+  isPageDocumentation2,
   isEndIndex
 };
 
@@ -222,10 +252,11 @@ extern int documentedHtmlFiles;
 extern int documentedPages;
 extern int documentedDirs;
 
-void startTitle(OutputList &ol,const char *fileName);
+void startTitle(OutputList &ol,const char *fileName,Definition *def=0);
 void endTitle(OutputList &ol,const char *fileName,const char *name);
 void startFile(OutputList &ol,const char *name,const char *manName,
-               const char *title,HighlightedItem hli=HLI_None);
+               const char *title,HighlightedItem hli=HLI_None,
+               bool additionalIndices=FALSE);
 void endFile(OutputList &ol,bool external=FALSE);
 
 void initClassMemberIndices();
@@ -235,4 +266,8 @@ void addClassMemberNameToIndex(MemberDef *md);
 void addFileMemberNameToIndex(MemberDef *md);
 void addNamespaceMemberNameToIndex(MemberDef *md);
 
+// search engine
+void writeJavascriptSearchIndex();
+void writeSearchCategories(FTextStream &t);
+
 #endif
